import pygame
from resource import absolute_path as abspath
import random

class Renderer(object):
    def __init__(self, size=None, windowTitle=None, fullscreenMode=False,
      debugMode=False, *args, **kwds):
        if size is None:
            raise TypeError('size not defined')
        if windowTitle is None:
            raise TypeError('windowTitle not defined')
        super(Renderer, self).__init__(*args, **kwds)
        pygame.font.init()
        self.fullscreen_mode = fullscreenMode
        self.debug_mode = debugMode
        self.init_display(windowTitle=windowTitle)
        self.screen = self.get_drawing_surface(size)
        self.clock = pygame.time.Clock()
        
        self.text_layout_margin = (-5, -5)
        self.line_spacing = -10
        
        self.word_spacing = 10

        self.fpsfont = pygame.font.Font(abspath('fonts/robots.ttf'), 24)
        self.font = pygame.font.Font(abspath('fonts/IndieFlower.ttf'), 24)
        self.font_color = (20, 20, 100)
        self.font_active_color = (80, 80, 255)

        # scene image
        self._scene = None
        self._text_rect = None
        self._image_rect = None
        self._new_scene = None
        self._scene_transition = 0

        # image
        self._image = None

    def dirty(self):
        return self._scene_transition > 0

    def init_display(self, windowTitle=None):
        if windowTitle is None:
            raise TypeError('windowTitle not defined')
        pygame.display.init()
        pygame.display.set_caption(windowTitle)
    
    def get_drawing_surface(self, size=None):
        if size is None:
            raise TypeError('size not defined')
        flags = 0
        if self.fullscreen_mode:
            flags = pygame.FULLSCREEN
        return pygame.display.set_mode(size, flags)
    
    def set_scene(self, new_scene):
        if self._scene is None:
            self._scene = pygame.image.load(abspath('images/white.png')).convert()
            self._text_rect = new_scene.text_rect
            self._image_rect = new_scene.image_rect
        
        self._new_scene = pygame.image.load(abspath(new_scene.image_file)).convert()
        self._text_rect = new_scene.text_rect
        self._image_rect = new_scene.image_rect
        self._scene_transition = 25
        self.screen.blit(self._scene, (0, 0))

    def render_scene(self):
        if self._new_scene:
            if self._scene_transition > 0:
                self.screen.blit(self._scene, (0, 0))
                self._new_scene.set_alpha(255/self._scene_transition)
                self.screen.blit(self._new_scene, (0, 0))
                self._scene_transition -= 1
            else:
                self._scene = self._new_scene
                self._new_scene = None
        else:
            self.screen.blit(self._scene, (0, 0))

    def set_image(self, new_image):
        self._image = pygame.image.load(abspath(new_image)).convert_alpha()

    def render_image(self):
        if self._image:
            img_width = self._image.get_width()
            img_height = self._image.get_height()
            if img_width > self._image_rect.width or img_height > self._image_rect.height:
                ratio_x = self._image_rect.width / float(img_width)
                ratio_y = self._image_rect.height / float(img_height)
                ratio = min(ratio_x, ratio_y)
                image = pygame.transform.smoothscale(self._image, 
                        (int(img_width*ratio), int(img_height*ratio)))
            else:
                image = self._image

            self.screen.blit(image, self._image_rect)

    def render_text(self, text, color):
        return self.font.render(text, True, color)
        
    def draw(self, allWords=None, story=None, update_all=False):
        self.clock.tick(60)
        if allWords is None:
            raise TypeError('allWords not defined')
        if story is None:
            raise TypeError('story not defined')

        if self.dirty() or update_all:
            self.render_scene()
            if self.debug_mode:
                fps_text = self.fpsfont.render("fps: %d percent: %d"%(
                  self.clock.get_fps() , story.percent()), False, (0,0,0))
                self.screen.blit(fps_text, (2,2))
            if self._scene_transition == 0:
                self.render_image()
                for word in allWords:
                    word.draw(self.screen)
            pygame.display.flip()
    
    def get_layout_rect(self):
        return self._text_rect.inflate(*self.text_layout_margin)
    
    def layout_sentence(self, sentenceWords=None):
        if sentenceWords is None:
            raise TypeError('sentenceWords not defined')
        #set line and position
        layout_rect = self.get_layout_rect()
        remaining_width = layout_rect.width
        y = 0
        
        for word in sentenceWords:
            word_width = word.width()
            if remaining_width < word_width:
                remaining_width = layout_rect.width
                y = y + word.height() + self.line_spacing
            word.set_pos((layout_rect.left
              + layout_rect.width - remaining_width, layout_rect.top + y))
            remaining_width = remaining_width - word_width \
              - self.word_spacing

        self.newword_top = y + sentenceWords[0].height() + self.line_spacing \
          + self.word_spacing
    
    def layout_new_words(self, sentenceWords=None, newSentenceWords=None):
        if sentenceWords is None:
            raise TypeError('sentenceWords not defined')
        if newSentenceWords is None:
            raise TypeError('newSentenceWords not defined')
        layout_rect = self.get_layout_rect()
        
        max_height = layout_rect.height - self.newword_top
        max_height = min(max_height, sentenceWords[0].height() * 1.5)
        top = layout_rect.top + self.newword_top
        
        for word in newSentenceWords:
            offset = (layout_rect.left
              + int(random.random() * (layout_rect.width - word.width())),
              top + int(random.random() * max_height))
            word.set_pos(offset)
            word.check_collisions(newSentenceWords)
            self.check_page(word.rect)
    
    def check_page(self, rect=None):
        if rect is None:
            raise TypeError('rect not defined')
        if not self._text_rect.contains(rect):
             if rect.top < self._text_rect.top:
                 rect.top = self._text_rect.top 
             elif rect.bottom > self._text_rect.bottom:
                 rect.bottom = self._text_rect.bottom
             if rect.left < self._text_rect.left:
                 rect.left = self._text_rect.left
             if rect.right > self._text_rect.right:
                 rect.right = self._text_rect.right
