from __future__ import with_statement
from itertools import takewhile
import re
import sys
import codecs

class StoryObject(object):
    def __init__(self,**kwargs):
        for (k,v) in kwargs.items():
            setattr(self,k,v)


class Word(StoryObject):
    """ A word is a word is a word"""
    def __init__(self,**kwargs):
        self.actions=[]
        self.word=""
        super(Word,self).__init__(**kwargs)
    
    def __str__(self):
        return "%s"%self.word
    
    def add_action(self,action):
        self.actions.append(action)
    
    def is_actionable(self):
        try:
                return len(self.actions)>0
        except NameError:
                """ no self.actions defined """
                return False
    
    def has_action(self,action):
        return action in [a['action'] for a in self.actions]
     
    def get_action(self,action):
        """ returns the action for the given name """
        ret=[]
        for a in self.actions:
            if a["action"]==action:
                ret.append(a)
        return ret        
    
    def get_outcome_for_action(self,action):
        if self.has_action(action):
            return self.get_action(action)[0]["outcome"]
        else:
            return None

    def do_action(self,action,new=None):
        if self.has_action(action):
            if action=="remove":
                return self.get_outcome_for_action(action)
            elif action=="replace":
                for a in self.get_action(action):
                    if new.word==a["new"]:
                        return a["outcome"]

class Sentence(StoryObject):
    """ Sentences consist of words """
    def __init__(self,**kwargs):
        self.words=[]
        self.newwords=[]
        self.outcomes=[]
        self.image=""
        self.scene=None
        self.visited=False
        super(Sentence,self).__init__(**kwargs)
    
    def add_word(self,word):
       """adds a word to the sentence """
       self.words.append(word)
    
    def add_newword(self,word): # HIER
        self.newwords.append(word)

    def get_word_by_string(self,wordstring):
        """ returns a word by the word string """
        for w in self.words:
            if (wordstring==w.word):
                return w
        sys.stderr.write("No such word %s\n" % wordstring)
        return Word(word="")
    
    def get_newwords(self):
        return (w for w in self.newwords)

    def __str__(self):
        return " ".join(["%s"%w for w in self.words])
    
    def __iter__(self):
        self.index=0
        return self
    
    def next(self):
        self.index+=1
        if self.index<=len(self.words):
            return self.words[self.index-1]
        else:
            raise StopIteration


class Story:
    def __init__(self, pfile=None):
        self.objects={}
        if pfile:
           self.pfile = pfile
           self.parse_sentences()
     
    def parse_sentences(self):
        for block in self.blocks():
            self.create_sentence(block)

    def create_sentence(self,block):
        if not len(block):
            return
        (id,words)=block.pop(0).split(" ",1)
        words=[Word(word=w) for w in words.split()]
        sentence=Sentence(words=words, id=id)
        self.add_object(id,sentence)
        if len(block):
            """ we still have blocks -> eventual newwords """
            [self.process_actions(sentence,l) for l in block]
     
    def process_actions(self,sentence,line):
        """ Processes /a/b/1.2 and adds an outcome to the sentence """
        if re.match("^image:",line):
                sentence.image=line.split(":")[1]
                return None
        if re.match("^scene:",line):
                sentence.scene=line.split(":")[1]
                return None
        try:
                (null,old,new,outcome)=line.split("/")
        except:
                sys.stderr.write(line + '\n')
        if new:
            if old:
                if len(new.split())>1:
                     (null,new)=new.split(" ",1)
                sentence.get_word_by_string(old).add_action({"action":"replace","new":new,"outcome":outcome})
            if new not in [w.word for w in sentence.newwords]:
                sentence.add_newword(Word(word=new,
                actions=[{"action":"move","outcome":None}]))
        else:
            sentence.get_word_by_string(old).add_action({"action":"remove","outcome":outcome})
             
    def blocks(self):
        with codecs.open(self.pfile, 'r', 'utf-8') as pfile:
            ret = []
            for line in pfile:
                line = self.stripline(line)

                if not line:
                    yield ret
                    ret = []
                else:
                    ret.append(line)
    
    def stripline(self,line):
        return line.split("#")[0].strip()

    
    def percent(self):
        objs=self.objects.values()
        i=0
        for o in objs:
            if o.visited:
               i+=1
        return i/float(len(objs))*100

    def get_object(self, id):
        """ Returns an object for a given ID """
        
        if id in self.objects:
            return self.objects[id]
        else:
            block = ['0.0 You have explored %d percent of this world. Start again?' %self.percent(),
            '/Start//1.1']
            self.create_sentence(block)
            return self.objects['0.0']

        
    def add_object(self, id, se):
        """ Adds an object to objects for a given ID """
        
        self.objects[id] = se
 
